subroutine bcheck2d(tmp,u,w,xp,zp,dx,dz,dxs,dzs,asp)
!***********************************************************************
!*     benchmark comparizon with Blankenbach et al. [1990]
!*     values to be calculated:
!*     (1) nusselt number
!*     (2) rms velocity
!*     (3) temperature gradients at corners
!*     (4) depth and values of extrema of temperature at x=asp/2
!***********************************************************************
  implicit none
  include "meshnum.h"
!c
  integer :: i,j
  real(kind=8),dimension(iax,jax),intent(in) :: tmp,u,w
  real(kind=8),dimension(iax),intent(in) :: dx,dxs,xp
  real(kind=8),dimension(jax),intent(in) :: dz,dzs,zp
  real(kind=8),intent(in) :: asp
!c
  integer :: im,istat1,istat2
  real(kind=8) :: hfn,tbot,tgd1,tgd2,tgd3,tgd4,tgrd,tsrf,ud1,vd2,vrms,wd1,dum
  real(kind=8),dimension(jax) :: tct
  real(kind=8) :: coef1,coef2,coef3,dum1,dum2,tex,tex1,tex2,zex,zex1,zex2
!c
!c ==========================
!c     (1) nusselt number
!c ==========================
!c
  tsrf = 0.d0
  tbot = 0.d0
  tgrd = 0.d0
  do i = 2,iax-1
     tbot = tbot + (tmp(i,1)+tmp(i,2))*dx(i)
     tgrd = tgrd + (tmp(i,jax-1)-tmp(i,jax))*dx(i)
     tsrf = tsrf + (tmp(i,jax-1)+tmp(i,jax))*dx(i)
  enddo
  tsrf = 0.5d0*tsrf/asp
  tbot = 0.5d0*tbot/asp
  tgrd = tgrd/asp/dzs(jax)
  hfn = tgrd/(tbot-tsrf)
!c
!c ========================
!c     (2) rms velocity
!c ========================
!c
  vrms = 0.d0
  do j = 2,jax-1
     do i = 2,iax-1
        ud1 = 0.5d0*(u(i,j)+u(i+1,j))
        wd1 = 0.5d0*(w(i,j)+w(i,j+1))
        vd2 = ud1**2+wd1**2
        vrms = vrms + vd2*dx(i)*dz(j)
     enddo
  enddo
  vrms = dsqrt(vrms/asp)
!c
!c ===========================================
!c     (3) temperature gradient at corners
!c ===========================================
!c
  tgd1 = (tmp(2,jax-1)-tmp(2,jax))/dzs(jax)
  tgd2 = (tmp(iax-1,jax-1)-tmp(iax-1,jax))/dzs(jax)
  tgd3 = (tmp(iax-1,1)-tmp(iax-1,2))/dzs(2)
  tgd4 = (tmp(2,1)-tmp(2,2))/dzs(2)
!c
!c ==================================================================
!c     (4) depths and values of extrema of temperature at x=asp/2
!c ==================================================================
!c
  do i = 2,iax-2
     dum = (xp(i)-.5d0*asp)*(xp(i+1)-.5d0*asp)
     if (dum.le.0.d0) im = i
  enddo
!c
  coef1 = (xp(im+1)-.5d0)/dxs(im+1)
  coef2 = (.5d0-xp(im))/dxs(im+1)
  do j = 1,jax
     tct(j) = tmp(im,j)*coef1+tmp(im+1,j)*coef2
  enddo
!c
!c     find extrema by local quadratic interpolation
!c
  tex1 = tct(jax-1)
  tex2 = tct(2)
  zex1 = zp(jax-1)
  zex2 = zp(2)
  istat1 = 1
  istat2 = 1
  do j = 2,jax-1
     coef1 = tct(j-1)/(zp(j-1)-zp(j))/(zp(j-1)-zp(j+1))
     coef2 = tct(j)/(zp(j)-zp(j-1))/(zp(j)-zp(j+1))
     coef3 = tct(j+1)/(zp(j+1)-zp(j-1))/(zp(j+1)-zp(j))
     dum1 = coef1*(2.d0*zp(j-1)-zp(j)-zp(j+1)) &
          +coef2*(zp(j-1)-zp(j+1)) &
          +coef3*(zp(j-1)-zp(j))
     dum2 = coef1*(zp(j+1)-zp(j)) &
          +coef2*(zp(j+1)-zp(j-1)) &
          +coef3*(2.d0*zp(j+1)-zp(j-1)-zp(j))
     if (dum1*dum2.le.0.d0) then
        zex = 0.5d0*(coef1*(zp(j)+zp(j+1))+coef2*(zp(j-1)+zp(j+1)) &
             +coef3*(zp(j-1)+zp(j)))/(coef1+coef2+coef3)
        tex = coef1*(zex-zp(j))*(zex-zp(j+1)) &
             +coef2*(zex-zp(j-1))*(zex-zp(j+1)) &
             +coef3*(zex-zp(j-1))*(zex-zp(j))
        if (tex.gt.tex1) then
           zex1 = zex
           tex1 = tex
           istat1 = 0
        endif
        if (tex.lt.tex2) then
           zex2 = zex
           tex2 = tex
           istat2 = 0
        endif
     endif
  enddo
!c
  write(*,'("================================================")')
  write(*,'("  Benchmark Test with Blankenbach et al.[1990]  ")')
  write(*,'("================================================")')
  write(*,*)"Nu   =",hfn
  write(*,*)"Vrms =",vrms
  write(*,*)"q1   =",tgd1
  write(*,*)"q2   =",tgd2
  write(*,*)"q3   =",tgd3
  write(*,*)"q4   =",tgd4
  if (istat1 .eq. 0) write(*,*)"Te1 =",tex1," at z =",zex1
  if (istat2 .eq. 0) write(*,*)"Te2 =",tex2," at z =",zex2
!c
  return
end subroutine bcheck2d
