subroutine hderef(u,w,psi,visc,dx,dz,dxs,dzs, &
     dxi,dzi,dxsi,dzsi,dtrm,bvlt,bvlb,ivlt,ivlb, &
     idcmp,apb)
!***********************************************************************
!*     calculate stream function
!*     under reflective side boudary condition
!*
!*     when called with for idcmp/=0, the setup/decomposition steps are
!*     skipped, by assuming the viscosity distributions is unchanged
!*
!*     GBHE is solved by Cholesky decomposition
!*     by calling LAPACK subroutines dpbtrf and dpbtrs
!***********************************************************************
  implicit none
  include "meshnum.h"
!c
  integer :: i,j,ij
  real(kind=8),dimension(iax),intent(in) :: dx,dxs,dxi,dxsi
  real(kind=8),dimension(jax),intent(in) :: dz,dzs,dzi,dzsi
  real(kind=8),dimension(iax,jax),intent(in) :: visc,dtrm
  real(kind=8),dimension(iax+1),intent(in) :: bvlt,bvlb
  integer,intent(in) :: ivlt,ivlb
  real(kind=8),dimension(iax,jax),intent(out) :: u,w
  real(kind=8),dimension(iax+1,jax+1),intent(out) :: psi
  real(kind=8),dimension(iax+1,jax+1,13) :: ap1,ap2
  real(kind=8),dimension(iax+1,jax+1) :: bp2
  real(kind=8),dimension(iax+1,jax+1,8) :: pd,qd
  real(kind=8),dimension(iax,jax) :: vcc,vcg
!c
!c     variables necessary for LAPACK subroutines
!c     nm: dimension of array
!c     m1: jax-3
!c     m2: upper band width
!c     lda: leading dimension (.ge.m2+1)
!c     apb(lda,nm): coefficient band matrix
!c     b(nm): RHS vector
!c     apb1(lda,1-m2:nm): work array
!c     info
!c
  integer,intent(inout) :: idcmp
  integer,parameter :: nm=(iax-3)*(jax-3),m1=jax-3,m2=2*m1,lda=m2+1
  integer :: info
  real(kind=8),dimension(lda,nm),intent(inout) :: apb
  real(kind=8),dimension(nm) :: bx
!c
!c ------------------------------------------
!c     calculate viscosity at grid points
!c ------------------------------------------
!c
!c     replace array
!c
  do j = 2,jax-1
     do i = 2,iax-1
        vcc(i,j) = visc(i,j)
     end do
  end do
!c
!c     set boundary condition
!c
  do j = 1,jax
     vcc(1,j) = visc(2,j)
     vcc(iax,j) = visc(iax-1,j)
  end do
  do i = 1,iax
     vcc(i,1) = visc(i,2)
     vcc(i,jax) = visc(i,jax-1)
  end do
!c
!c     set viscosity at grid points
!c
  do j = 2,jax
     do i = 2,iax
        vcg(i,j) = 0.25d0*dxsi(i)*dzsi(j)* &
             (vcc(i-1,j-1)*dx(i-1)*dz(j-1) &
             +vcc(i,j-1)*dx(i)*dz(j-1) &
             +vcc(i-1,j)*dx(i-1)*dz(j) &
             +vcc(i,j)*dx(i)*dz(j))
     end do
  end do
!c
!c -----------------------------------------------------
!c     set coefficient matrix and boundary condition
!c -----------------------------------------------------
!c
!c     set coefficient matrix of psi(i,j) and rhs vector
!c
  do j = 3,jax-1
     do i = 3,iax-1
        pd(i,j,1) = -vcg(i,j-1)*dxs(i)*dzi(j-1)*dzsi(j-1)
        pd(i,j,2) = (-2.d0*vcc(i-1,j-1)+vcg(i-1,j))*dxi(i-1)
        pd(i,j,3) = &
             2.d0*vcc(i,j-1)*dxi(i)+2.d0*vcc(i-1,j-1)*dxi(i-1) &
             +vcg(i,j-1)*dxs(i)*dzi(j-1)*dzsi(j-1) &
             +vcg(i,j)*((dzi(j)+dzi(j-1))*dxs(i)*dzsi(j) &
             -(dxi(i)+dxi(i-1)))
        pd(i,j,4) = (-2.d0*vcc(i,j-1)+vcg(i+1,j))*dxi(i)
        pd(i,j,5) = (2.d0*vcc(i-1,j)-vcg(i-1,j))*dxi(i-1)
        pd(i,j,6) = &
             -2.d0*vcc(i-1,j)*dxi(i-1)-2.d0*vcc(i,j)*dxi(i) &
             -vcg(i,j)*((dzi(j)+dzi(j-1))*dxs(i)*dzsi(j) &
             -(dxi(i)+dxi(i-1))) &
             -vcg(i,j+1)*dxs(i)*dzi(j)*dzsi(j+1)
        pd(i,j,7) = (2.d0*vcc(i,j)-vcg(i+1,j))*dxi(i)
        pd(i,j,8) = vcg(i,j+1)*dxs(i)*dzi(j)*dzsi(j+1)
!c
        qd(i,j,1) = (2.d0*vcc(i-1,j-1)-vcg(i,j-1))*dzi(j-1)
        qd(i,j,2) = (-2.d0*vcc(i,j-1)+vcg(i,j-1))*dzi(j-1)
        qd(i,j,3) = vcg(i-1,j)*dzs(j)*dxi(i-1)*dxsi(i-1)
        qd(i,j,4) = &
             -2.d0*vcc(i-1,j)*dzi(j)-2.d0*vcc(i-1,j-1)*dzi(j-1) &
             -vcg(i-1,j)*dzs(j)*dxi(i-1)*dxsi(i-1) &
             -vcg(i,j)*((dxi(i)+dxi(i-1))*dzs(j)*dxsi(i) &
             -(dzi(j)+dzi(j-1)))
        qd(i,j,5) = &
             2.d0*vcc(i,j-1)*dzi(j-1)+2.d0*vcc(i,j)*dzi(j) &
             +vcg(i,j)*((dxi(i)+dxi(i-1))*dzs(j)*dxsi(i) &
             -(dzi(j)+dzi(j-1))) &
             +vcg(i+1,j)*dzs(j)*dxi(i)*dxsi(i+1)
        qd(i,j,6) = -vcg(i+1,j)*dzs(j)*dxi(i)*dxsi(i+1)
        qd(i,j,7) = (2.d0*vcc(i-1,j)-vcg(i,j+1))*dzi(j)
        qd(i,j,8) = (-2.d0*vcc(i,j)+vcg(i,j+1))*dzi(j)
     enddo
  enddo
!c
  do j = 3,jax-1
     do i = 3,iax-1
        ap1(i,j, 1) = qd(i,j,3)*dxi(i-2)
        ap1(i,j, 2) = -pd(i,j,2)*dzi(j-1)+qd(i,j,1)*dxi(i-1)
        ap1(i,j, 3) = pd(i,j,2)*dzi(j-1)-pd(i,j,5)*dzi(j) &
             -qd(i,j,3)*dxi(i-2)+qd(i,j,4)*dxi(i-1)
        ap1(i,j, 4) = pd(i,j,5)*dzi(j)+qd(i,j,7)*dxi(i-1)
        ap1(i,j, 5) = -pd(i,j,1)*dzi(j-2)
        ap1(i,j, 6) = pd(i,j,1)*dzi(j-2)-pd(i,j,3)*dzi(j-1) &
             -qd(i,j,1)*dxi(i-1)+qd(i,j,2)*dxi(i)
        ap1(i,j, 7) = pd(i,j,3)*dzi(j-1)-pd(i,j,6)*dzi(j) &
             -qd(i,j,4)*dxi(i-1)+qd(i,j,5)*dxi(i)
        ap1(i,j, 8) = pd(i,j,6)*dzi(j)-pd(i,j,8)*dzi(j+1) &
             -qd(i,j,7)*dxi(i-1)+qd(i,j,8)*dxi(i)
        ap1(i,j, 9) = pd(i,j,8)*dzi(j+1)
        ap1(i,j,10) = -pd(i,j,4)*dzi(j-1)-qd(i,j,2)*dxi(i)
        ap1(i,j,11) = pd(i,j,4)*dzi(j-1)-pd(i,j,7)*dzi(j) &
             -qd(i,j,5)*dxi(i)+qd(i,j,6)*dxi(i+1)
        ap1(i,j,12) = pd(i,j,7)*dzi(j)-qd(i,j,8)*dxi(i)
        ap1(i,j,13) = -qd(i,j,6)*dxi(i+1)
     end do
  end do
!c
  if (idcmp .eq. 0) then
!c
!c -------------------------------
!c     set coefficient matrix
!c -------------------------------
!c
     do ij = 1,13
        do j = 1,jax+1
           do i = 1,iax+1
              ap2(i,j,ij) = 0.d0
           end do
        end do
     end do
     do ij = 1,13
        do j = 3,jax-1
           do i = 3,iax-1
              ap2(i,j,ij) = ap1(i,j,ij)
           end do
        end do
     end do
!c
!c ------------------------------
!c     set boundary condition
!c ------------------------------
!c
!c     top boundary (j = jax-1)
!c     case 1 : impermeable, free slip
!c
     if (ivlt .eq. 1) then
        do i = 3,iax-1
           ap2(i,jax-1, 7) = ap2(i,jax-1,7)-ap1(i,jax-1,9)
           ap2(i,jax-1, 4) = 0.d0
           ap2(i,jax-1, 8) = 0.d0
           ap2(i,jax-1,12) = 0.d0
           ap2(i,jax-1, 9) = 0.d0
           ap2(i,jax-2, 9) = 0.d0
        end do
     end if
!c
!c     top boundary (j = jax-1)
!c     case 2 : impermeable, velocity
!c
     if (ivlt .eq. 2) then
        do i = 3,iax-1
           ap2(i,jax-1, 7) = ap2(i,jax-1,7)+ap1(i,jax-1,9)
           ap2(i,jax-1, 4) = 0.d0
           ap2(i,jax-1, 8) = 0.d0
           ap2(i,jax-1,12) = 0.d0
           ap2(i,jax-1, 9) = 0.d0
           ap2(i,jax-2, 9) = 0.d0
        end do
     end if
!c
!c     bottom boundary (j = 3)
!c     case 1 : impermeable, free slip
!c
     if (ivlb .eq. 1) then
        do i = 3,iax-1
           ap2(i,3, 7) = ap2(i,3,7)-ap1(i,3,5)
           ap2(i,3, 2) = 0.d0
           ap2(i,3, 6) = 0.d0
           ap2(i,3,10) = 0.d0
           ap2(i,3, 5) = 0.d0
           ap2(i,4, 5) = 0.d0
        end do
     end if
!c
!c     bottom boundary (j = 3)
!c     case 2 : impermeable, velocity
!c
     if (ivlb .eq. 2) then
        do i = 3,iax-1
           ap2(i,3, 7) = ap2(i,3,7)+ap1(i,3,5)
           ap2(i,3, 2) = 0.d0
           ap2(i,3, 6) = 0.d0
           ap2(i,3,10) = 0.d0
           ap2(i,3, 5) = 0.d0
           ap2(i,4, 5) = 0.d0
        end do
     end if
!c
!c     side boundary (i = 3)
!c     impermeable, free slip
!c
     do j = 3,jax-1
        ap2(3,j,7) = ap2(3,j,7)-ap1(3,j,1)
        ap2(3,j,2) = 0.d0
        ap2(3,j,3) = 0.d0
        ap2(3,j,4) = 0.d0
        ap2(3,j,1) = 0.d0
        ap2(4,j,1) = 0.d0
     end do
!c
!c     side boundary (i = iax-1)
!c     impermeable, free slip
!c
     do j = 3,jax-1
        ap2(iax-1,j, 7) = ap2(iax-1,j,7)-ap1(iax-1,j,13)
        ap2(iax-1,j,10) = 0.d0
        ap2(iax-1,j,11) = 0.d0
        ap2(iax-1,j,12) = 0.d0
        ap2(iax-1,j,13) = 0.d0
        ap2(iax-2,j,13) = 0.d0
     end do
!c
!c ///////////////////////////////
!c     setup biharmonic solver
!c ///////////////////////////////
!c
!c --------------------------------
!c     input coefficient matrix
!c --------------------------------
!c
!c     reset array before input
!c
     do j = 1,nm
        do i = 1,lda
           apb(i,j) = 0.d0
        enddo
     enddo
     do j = 3,jax-1
        do i = 3,iax-1
           ij = (i-3)*(jax-3)+j-2
           apb(1,   ij) = ap2(i  ,j  ,7) ! (I  ,J  )
           apb(2,   ij) = ap2(i  ,j+1,6) ! (I  ,J-1)
           apb(3,   ij) = ap2(i  ,j+2,5) ! (I  ,J-2)
           apb(m1,  ij) = ap2(i+1,j-1,4) ! (I-1,J+1)
           apb(m1+1,ij) = ap2(i+1,j  ,3) ! (I-1,J  )
           apb(m1+2,ij) = ap2(i+1,j+1,2) ! (I-1,J-1)
           apb(m2+1,ij) = ap2(i+2,j  ,1) ! (I-2,J  )
        enddo
     enddo
!c
!c     Cholesky decomposition
!c
     call dpbtrf("L",nm,m2,apb,lda,info)
!c
     idcmp = 1
!c$$$      else
!c$$$         write(*,*)"Decomposition skipped"
  endif
!c
!c ------------------------
!c     input rhs vector
!c ------------------------
!c
  do j = 1,jax+1
     do i = 1,iax+1
        bp2(i,j) = 0.d0
     end do
  end do
  do j = 3,jax-1
     do i = 3,iax-1
        bp2(i,j) = 0.5d0*(dz(j)*(dtrm(i,j-1)-dtrm(i-1,j-1))+ &
             dz(j-1)*(dtrm(i,j)-dtrm(i-1,j)))
     end do
  end do
!c
!c     top boundary (j = jax-1)
!c     case 2 : impermeable, velocity
!c
  if (ivlt .eq. 2) then
     do i = 3,iax-1
        bp2(   i,jax-1) = bp2(i,jax-1)+bvlt(i)*ap1(i,jax-1,9)
!c     bvlt(i) == -2*vx(i)*dz
     end do
  end if
!c
!c     bottom boundary (j = 3)
!c     case 2 : impermeable, velocity
!c
  if (ivlb .eq. 2) then
     do i = 3,iax-1
        bp2(   i,3) = bp2(i,3)+bvlb(i)*ap1(i,3,5)
!c     bvlb(i) == -2*vx(i)*dz
     end do
  end if
  do ij = 1,nm
     bx(ij) = 0.d0
  end do
  do j = 3,jax-1
     do i = 3,iax-1
        ij = (i-3)*(jax-3) + j - 2
        bx(ij) = bp2(i,j)
     end do
  end do
!c
  call dpbtrs("L",nm,m2,1,apb,lda,bx,nm,info)
!c
!c /////////////////////////////
!c     end biharmonic solver
!c /////////////////////////////
!c
!c ----------------------------
!c     substitute solutions
!c ----------------------------
!c
  do j = 1,jax+1
     do i = 1,iax+1
        psi(i,j) = 0.d0
     end do
  end do
  do j = 3,jax-1
     do i = 3,iax-1
        ij = (i-3)*(jax-3)+j-2
        psi(i,j) = bx(ij)
     end do
  end do
!c
!c ----------------------------------------------
!c     set top and bottom boundary conditions
!c ----------------------------------------------
!c
!c     stress free
!c
  if (ivlt .eq. 1) then
     do i = 3,iax-1
        psi(i,jax+1) = -psi(i,jax-1)
     end do
  end if
  if (ivlb .eq. 1) then
     do i = 3,iax-1
        psi(i,1) = -psi(i,3)
     end do
  end if
!c
!c     velocity boundary
!c
  if (ivlt .eq. 2) then
     do i = 3,iax-1
        psi(i,jax+1) = psi(i,jax-1) - bvlt(i)
     end do
  end if
  if (ivlb .eq. 2) then
     do i = 3,iax-1
        psi(i,1) = psi(i,3) - bvlb(i)
     end do
  end if
!c
!c     set side boundary condition
!c
!c     stress free
!c
  do j = 1,jax+1
     psi(1,j) = -psi(3,j)
     psi(iax+1,j) = -psi(iax-1,j)
  end do
!c
!c --------------------------
!c     calculate velocity
!c --------------------------
!c
  do j = 1,jax
     do i = 1,iax
        u(i,j) = (psi(i,j+1)-psi(i,j))*dzi(j)
        w(i,j) = -(psi(i+1,j)-psi(i,j))*dxi(i)
     end do
  end do
!c
  return
end subroutine hderef
