subroutine init(asp,dx,dz,dxs,dzs,xp,zp,xc,zc, &
     tmp,q,tsurf,tbot,tamp,bvlt,bvlb,ivlt,ivlb, &
     siv,eiv,visc,icont)
!***********************************************************************
!*     set initial conditions
!*
!*     icont: = 0 if new run
!*            = 1 if successive run
!*     ivlt: = 1 if free-slip top surface
!*           = 2 if fixed velocity top surface
!*     ivlb: = 1 if free-slip bottom surface
!*           = 2 if fixed velocity bottom surface
!*     input
!*     output
!*        tmp: temperature field
!*        q: internal heating rate
!*        tamp: reference temperature
!*        bvlt: -2*(surface velocity)*dz
!*        bvlb: -2*(bottom  velocity)*dz
!***********************************************************************
  implicit none
  include "meshnum.h"
!c
  integer :: i,j
  integer,intent(in) :: icont,ivlt,ivlb
  real(kind=8),dimension(iax,jax),intent(out) :: tmp,q,visc,eiv,siv
  real(kind=8),dimension(iax+1),intent(out) :: bvlt,bvlb
  real(kind=8),dimension(iax+1,jax+1) :: psi
  real(kind=8),intent(in) :: asp,tsurf,tbot
  real(kind=8),intent(inout) :: tamp
  real(kind=8),dimension(iax),intent(in) :: dx,dxs,xp,xc
  real(kind=8),dimension(jax),intent(in) :: dz,dzs,zp,zc
  real(kind=8) :: cell,dtbl,zz,t1,t2,tm
  real(kind=8),parameter :: pi = 4.d0*datan(1.d0)
!c
  if (icont .eq. 0) then
!c
!c =============================
!c     for new run (icont=0)
!c =============================
!c
!c -------------------------------
!c     set initial temperature
!c -------------------------------
!c
     cell = 1.d0
!!$     cell = 4.d0
     tm = 0.5d0*(tsurf+tbot)
!!$     dtbl = .1d0
     dtbl = .5d0
     do j = 2,jax-1
        zz = zp(j)
        t1 = tsurf+(1.d0-zz)*(tm-tsurf)/dtbl
        t2 = dmax1(tbot+zz*(tm-tbot)/dtbl,tm)
        do i = 2,iax-1
           tmp(i,j) = dmin1(t1,t2) &
                + tamp*dcos(pi*cell*xp(i)/asp)*dsin(pi*zz)
        enddo
     enddo
!c
     do i = 2,iax-1
        tmp(i,1) = 2.d0*tbot - tmp(i,2)
        tmp(i,jax) = 2.d0*tsurf - tmp(i,jax-1)
     enddo
     do j = 1,jax
        tmp(1,j) = tmp(2,j)
        tmp(iax,j) = tmp(iax-1,j)
     enddo
!c
!c ----------------------------------------------------------
!c     set velocity boundary condition for given velocity
!c ----------------------------------------------------------
!c
     if (ivlt .eq. 2) then
        do i = 1,iax+1
           bvlt(i) = 0.d0
        end do
     end if
     if (ivlb .eq. 2) then
        do i = 1,iax+1
           bvlb(i) = 0.d0
        end do
     end if
!c
!c ----------------------------------------
!c     set initial heat_production_rate
!c ----------------------------------------
!c
     do j = 2,jax-1
        do i = 2,iax-1
           q(i,j) = 0.d0
        enddo
     enddo
!c
     tamp = tbot - tsurf
!c
  else if (icont .eq. 1) then
!c
!c ================================
!c     successive run (icont=1)
!c ================================
!c
     read(9)tmp,psi,q,siv,eiv,visc
!c
     do i = 1,iax+1
        bvlt(i) = psi(i,jax-1)-psi(i,jax+1)
     enddo
     do i = 1,iax+1
        bvlb(i) = psi(i,3)-psi(i,1)
     enddo
  endif
!c
  return
end subroutine init
