program main
!***********************************************************************
!*     test program for scalar processor
!***********************************************************************
  implicit none
  include "meshnum.h"
!c
  integer :: i,j

  real(kind=8),dimension(iax,jax) :: u,w,tmp,visc,buoy,sch,siv,eiv
  real(kind=8),dimension(iax+1,jax+1) :: psi

  real(kind=8),dimension(iax) :: dx,dxs,dxi,dxsi,xp,xc
  real(kind=8),dimension(jax) :: dz,dzs,dzi,dzsi,zp,zc
  real(kind=8) :: rat,asp,tsurf,tbot,tamp,qbot,vboti,time,dt,crmax
  real(kind=8) :: etl,etn,epl,epn
  integer :: nt,ntime,maxtime,iplot,iprnt,nstep
  integer :: ivlt,ivlb,icont,ipt,ivis
  real(kind=8),dimension(iax+1) :: bvlt,bvlb
  real(kind=8),dimension(iax,jax) :: sivo,eivo,viso,areai
  character(len=80) :: ipf,opf,file9,file10,file20
  integer :: idcmp
  real(kind=8),dimension(2*(jax-3)+1,(iax-3)*(jax-3)) :: arhde
!c
!c ------------------
!c     open files
!c ------------------
!c
  read(*,'(a)')ipf
  read(*,'(a)')opf
  read(*,'(a)')file9
  read(*,'(a)')file10
  read(*,'(a)')file20
  open(21,file=ipf)
  open(22,file=opf)
  open(9,file=file9,form="unformatted")
  open(10,file=file10,form="unformatted")
  open(20,file=file20)
!c
  call rdata(maxtime,nstep,iplot,iprnt,time,ivlt,ivlb,icont,ivis, &
       ipt,etl,etn,epl,epn,rat,tsurf,tbot,tamp,vboti,asp,crmax)
  call dfmesh(dx,dz,dxs,dzs,asp,xp,zp,xc,zc)
  call mconst(dx,dxs,dz,dzs,xp,xc,zp,zc &
       ,dxi,dxsi,dzi,dzsi,areai)
  call init(asp,dx,dz,dxs,dzs,xp,zp,xc,zc, &
       tmp,sch,tsurf,tbot,tamp,bvlt,bvlb,ivlt,ivlb, &
       siv,eiv,visc,icont)
  if ( (iplot .ne. 0) .and. &
       (icont .eq. 0) ) then
     call dxout(tmp,psi,u,w,xc,zc,dx,dz,dxi,dzi,0)
     call ppmout(tmp,xc,zc,0)
  endif
!c
!c ==========================
!c     begin time marching
!c ==========================
!c
  idcmp = 0
!c
  do nt = 1,maxtime
     ntime = nt + nstep
     do j = 1,jax
        do i = 1,iax
           eivo(i,j) = eiv(i,j)
           sivo(i,j) = siv(i,j)
           viso(i,j) = visc(i,j)
        enddo
     enddo
!c
!c ----------------------------------------------
!c     calculate buoyancy term at mesh points
!c ----------------------------------------------
!c
     call buoyant(tmp,rat,buoy)
!c
!c -----------------------------------
!c     solve hydrodynamic equation
!c -----------------------------------
!c
     if (etl**2 .gt. 1.d-20) idcmp = 0
     call calvisc(tmp,zp,etl,epl,visc)
     call hderef(u,w,psi,visc,dx,dz,dxs,dzs, &
          dxi,dzi,dxsi,dzsi,buoy,bvlt,bvlb,ivlt,ivlb,idcmp,arhde)
     call caleiv(u,w,eiv,siv,visc,dxs,dzs,dxi,dzi,dxsi,dzsi)
!c
!c ------------------------------------
!c     solve heat transfer equation
!c ------------------------------------
!c
     call tstep(u,w,dx,dz,dxs,dzs,siv,eiv,dt)
     call htrns(tmp,u,w,sch,dx,dz,dxs,dzs,asp, &
          tsurf,tbot,qbot,vboti,dt,crmax)
     time = time + dt
!c
!c     for printing
!c
     if ( (iprnt .ne. 0) .and. &
          (mod(nt,iprnt) .eq. 0) ) then
        call plot(asp,tmp,tsurf,tbot,dx,dz,u,w,eiv,siv,psi,time,ntime)
     endif
!c
!c     for drawing
!c
     if ( (iplot .ne. 0) .and. &
          (mod(nt,iplot) .eq. 0) ) then
        call dxout(tmp,psi,u,w,xc,zc,dx,dz,dxi,dzi,ntime/iplot)
        call ppmout(tmp,xc,zc,ntime/iplot)
        call checkv(ntime,tmp,siv,eiv,dx,dz,asp)
     endif
!c
  enddo
!c
!c =========================
!c     end time marching
!c =========================
!c
  call wdata(maxtime,nstep,iplot,iprnt,time,ivlt,ivlb, &
       ipt,etl,etn,epl,epn,rat,tsurf,tbot,tamp,vboti,asp,crmax)
!c
!c ----------------------
!c     output results
!c ----------------------
!c
  call bcheck2d(tmp,u,w,xp,zp,dx,dz,dxs,dzs,asp)
!c
!c     for a next run
!c
  write(10)tmp,psi,sch,siv,eiv,visc
  if ( (iplot .ne. 0) .and. &
       (mod(ntime,iplot) .ne. 0) ) then
     call checkv(ntime,tmp,siv,eiv,dx,dz,asp)
  endif
  if ( (iprnt .ne. 0) .and. &
       (mod(nt-1,iprnt) .ne. 0) ) then
     call plot(asp,tmp,tsurf,tbot,dx,dz,u,w,eiv,siv,psi,time,ntime-1)
  endif
!c
  stop
end program main
