#!/bin/csh -f
set data = ./$$.data
set gnuplot = ./$$.gnuplot

if ($#argv != 0) then
  set file = ( $argv[1-$#argv] )
else
  set file = ( 20*dat* )
endif

# convert 20*.dat
touch $data
foreach f ( $file[*] )
    switch ($f)
        case *.Z:
            zcat $f >> $data
            breaksw
        case *.gz:
            gzip -dc $f >> $data
            breaksw
        case *.bz2:
            bzip2 -dc $f >> $data
            breaksw
        default:
            cat $f >> $data
            breaksw
    endsw
end

set num = `wc -l $data`
cat $data | awk '{print $1,$2}' > t_rms.dat
cat $data | awk '{print $1,$3}' > t_plt.dat
cat $data | awk '{print $1,$4}' > t_tav.dat
cat $data | awk '{print $1,$5}' > t_hfs.dat
cat $data | awk '{print $1,$6}' > t_hfb.dat
cat $data | awk '{print $1,$7}' > t_bot.dat
cat $data | awk '{print $1,$8}' > t_smx.dat
cat $data | awk '{print $1,$9}' > t_emx.dat
cat $data | awk '{print $1,$10}' > t_psi.dat
echo $num[1] "data were processed."
rm -f $data

if ($?DISPLAY) then
cat <<EOF > ${gnuplot}
set autoscale y
set nologscale y
set xlabel "time"
set autoscale y
set ylabel "velocity"
plot 't_rms.dat' with lines
pause -1

set ylabel "temperature"
plot 't_tav.dat' with lines,'t_bot.dat' with lines
pause -1
set ylabel "heat flow"
plot 't_hfs.dat' with lines, 't_hfb.dat' with lines
pause -1
set autoscale y
set ylabel ""
plot 't_psi.dat' with lines
pause -1

set autoscale y
set ylabel "velocity"
plot 't_rms.dat' with lines,'t_plt.dat' with lines
pause -1
set ylabel "stress"
plot 't_smx.dat' with lines
pause -1
set ylabel "strain rate"
plot 't_emx.dat' with lines
set nolog y
set ylabel ""
pause -1
EOF
gnuplot ${gnuplot}
rm -f t_*.dat ${gnuplot}
endif
